@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                   Copyright (C) 2007  QRW Software
+--                 Copyright (C) 2007-2008  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,9 @@
 --
 --
 -- $Log: add_password.c,v $
+-- Revision 1.8  2008/02/03 01:00:15  jsedwards
+-- Changed DEFAULT_TYPE_RW to READ_WRITE.
+--
 -- Revision 1.7  2007/10/07 03:43:50  jsedwards
 -- Renamed 'nwos_set_block_estimate' to 'nwos_check_blocks_available' and
 -- changed to exit gracefully if not enough space available.
@@ -96,7 +99,7 @@ int main(int argc, char* argv[])
     memcpy(private_root_obj_ref.id, big_key+20, 4);
     serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
 
-    nwos_initialize_objectify(bf_key, linear, serial, DEFAULT_TYPE_RW, DEFAULT_FILE);
+    nwos_initialize_objectify(bf_key, linear, serial, READ_WRITE, DEFAULT_FILE);
 
     nwos_allocate_new_chunk(nwos_ref_to_word(&private_root_obj_ref));
 
