@@ -30,6 +30,48 @@
 --   into an adjacent block to free up that chunk.
 --
 -- $Log: eliminate_one_block_chunks.c,v $
+-- Revision 1.19  2007/12/09 16:38:42  jsedwards
+-- Added code to modify the used_chunks in the header when copying file.
+--
+-- Revision 1.18  2007/12/09 16:00:03  jsedwards
+-- Fix bug where first block wasn't being written to output file.
+--
+-- Revision 1.17  2007/12/09 13:35:44  jsedwards
+-- Added code to recalculate crc32 block checksum.
+--
+-- Revision 1.16  2007/12/09 00:52:06  jsedwards
+-- Added code to verify files.
+--
+-- Revision 1.15  2007/12/08 22:44:55  jsedwards
+-- Remove unused variables and code.
+--
+-- Revision 1.14  2007/12/08 22:39:29  jsedwards
+-- Moved objectify initialization to main so it can be done again for testing.
+--
+-- Revision 1.13  2007/12/08 21:59:18  jsedwards
+-- Added function to copy compressed file to output file changing as we go.
+--
+-- Revision 1.12  2007/12/08 20:53:47  jsedwards
+-- Added function to encrypt the reference list blocks that have changed.
+--
+-- Revision 1.11  2007/12/08 18:48:27  jsedwards
+-- Added function to put the new references in the reference list blocks.
+--
+-- Revision 1.10  2007/12/08 17:59:44  jsedwards
+-- Change to store the reference list block instead of the file reference.
+--
+-- Revision 1.9  2007/12/08 16:47:57  jsedwards
+-- Fixed find_new_locations to deal with multiple blocks in a row.
+--
+-- Revision 1.8  2007/12/07 15:06:08  jsedwards
+-- Changed locate function to print out stats of each found block.
+--
+-- Revision 1.7  2007/12/06 15:19:08  jsedwards
+-- First code to handle more than one block in a row.
+--
+-- Revision 1.6  2007/12/06 14:08:46  jsedwards
+-- Added function to find location to move block - incomplete.
+--
 -- Revision 1.5  2007/12/02 15:50:06  jsedwards
 -- Added counting the number of files checked and verify the block count is
 -- correct for each file.
@@ -64,6 +106,7 @@
 */
 
 #include <ctype.h>
+#include <openssl/blowfish.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -82,7 +125,8 @@ int blocks_found = 0;
 int first_block_blocks = 0;
 int extra_block_blocks = 0;
 int files_checked = 0;
-
+int files_found = 0;
+int refs_changed = 0;
 
 struct
 {
@@ -90,9 +134,18 @@ struct
     uint32 single;
     uint32 subsequent;
     uint32 new;
-    ObjRef file;
+    Ref_List_First_Block* first_block;
+    Ref_List_Extra_Block* extra_block;
 } blocks[4096];
 
+struct
+{
+    ObjRef file;
+    int seq;
+    Ref_List_First_Block* first_block;
+    Ref_List_Extra_Block* extra_block;
+} files[64];
+
 
 void find_single_block_chunks(char* compressed_file_name)
 {
@@ -251,19 +304,6 @@ static void check_ref_list_data(ObjRef* ref, uint8* data, size_t data_size, uint
 {
     uint8 computed[4];
 
-#if 0
-    printf("Total size: %d\n", total_size);
-    printf("ObjectHeader: %d\n", sizeof(EveryObject));
-    printf("object pointer: %p\n", object);
-    {
-      int i;
-      uint8* ptr = (uint8*)object + sizeof(EveryObject);
-      printf("data pointer: %p\n", ptr);
-      printf("data: ");
-      for (i = 0; i < data_size; i++) printf("%02x%c", *ptr++, i%16==15?'\n':' ');
-      printf("\n");
-    }
-#endif
 
     nwos_crc32_calculate((uint8*)data, data_size, computed);
 
@@ -296,19 +336,11 @@ static bool check_file(ObjRef* file_ref)
     uint32 num_blocks;
     uint32 block_count = 0;
     C_struct_File file_obj;
-    C_struct_Path_And_File_Association assoc_obj;
-//    DataObject data_obj;
-    struct timeval tv[2];      /* [0] is access time and [1] is modification time, see man utimes */
-//    MD5_CTX md5_context;           /* MD5 checksum context */
-//    struct sha1_ctx sha1_context;
-    uint8 md5_digest[16];
-    uint8 sha1_digest[20];
-    C_struct_MD5sum md5_object;
-    C_struct_SHA1sum sha1_object;
     uint8 ivec[IVEC_SIZE];
-    uint8 chksum[4];
     Ref_List_First_Block first_block;
     Ref_List_Extra_Block extra_block;
+    Ref_List_First_Block* first_block_ptr = NULL;
+    Ref_List_Extra_Block* extra_block_ptr = NULL;
     int block_index;
     int seq;
     ObjRef next_ref;
@@ -334,6 +366,8 @@ static bool check_file(ObjRef* file_ref)
 
 	num_blocks = (file_length + DATA_STORAGE_SIZE - 1) / DATA_STORAGE_SIZE;
 
+	memset(&first_block, 0, sizeof(first_block));
+
 	nwos_read_object_from_disk_and_decrypt(&file_obj.block_list, &first_block.list, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[0]);
 
 	for (i = 0; i < MAX_REFS_IN_REF_LIST; i++)
@@ -344,11 +378,28 @@ static bool check_file(ObjRef* file_ref)
 
 	    if (block_index >= 0)
 	    {
-		assert(is_void_reference(&blocks[block_index].file));
+		assert(blocks[block_index].first_block == NULL && blocks[block_index].extra_block == NULL);
 
-		copy_reference(&blocks[block_index].file, file_ref);
+		if (first_block_ptr == NULL)   /* this is the first one we've found */
+		{
+		    assert(files_found < 64);
 
-		printf("block: %08x  file: %08x\n", blocks[block_index].single, nwos_ref_to_word(file_ref));
+		    copy_reference(&files[files_found].file, file_ref);
+
+		    first_block_ptr = malloc(sizeof(first_block));
+
+		    assert(first_block_ptr != NULL);
+
+		    memcpy(first_block_ptr, &first_block, sizeof(first_block));
+
+		    files[files_found].first_block = first_block_ptr;
+
+		    files_found++;
+		}
+
+		blocks[block_index].first_block = first_block_ptr;
+
+		printf("block: %08x  first_block: %p\n", blocks[block_index].single, blocks[block_index].first_block);
 
 		blocks_found++;
 	    }
@@ -379,12 +430,41 @@ static bool check_file(ObjRef* file_ref)
 		    extra_block_blocks++;
 		}
 
+		memset(&extra_block, 0, sizeof(extra_block));
+
+		memcpy(&extra_block.ivec, ivec, sizeof(&extra_block.ivec));  /* save this ivec in case this block has to be written back to disk */
 		nwos_read_object_from_disk_and_decrypt(&next_ref,
 						       &extra_block.dirty,
 						       FILE_BLOCK_SIZE, 
 						       ivec,
 						       nwos_random_sequence[seq]);
 
+		if (first_block_ptr != NULL || extra_block_ptr != NULL)   /* already found one in this list */
+		{
+		    if (extra_block_ptr == NULL)   /* there was not a previous extra block */
+		    {
+			assert(first_block_ptr->next_block_ptr == NULL);
+
+			first_block_ptr->next_block_ptr = malloc(sizeof(extra_block));
+
+			assert(first_block_ptr->next_block_ptr != NULL);
+
+			extra_block_ptr = first_block_ptr->next_block_ptr;
+		    }
+		    else   /* there was a previous block */
+		    {
+			assert(extra_block_ptr->next_block_ptr == NULL);
+
+			extra_block_ptr->next_block_ptr = malloc(sizeof(extra_block));
+
+			assert(extra_block_ptr->next_block_ptr != NULL);
+
+			extra_block_ptr = extra_block_ptr->next_block_ptr;
+		    }
+
+		    memcpy(extra_block_ptr, &extra_block, sizeof(extra_block));
+		}
+
 		for (i = 0; i < MAX_REFS_IN_SIDECAR; i++)
 		{
 		    if (is_void_reference(&extra_block.refs[i])) break;
@@ -393,11 +473,31 @@ static bool check_file(ObjRef* file_ref)
 
 		    if (block_index >= 0)
 		    {
-			assert(is_void_reference(&blocks[block_index].file));
+			assert(blocks[block_index].first_block == NULL && blocks[block_index].extra_block == NULL);
 
-			copy_reference(&blocks[block_index].file, file_ref);
+			if (extra_block_ptr == NULL)   /* this is the first one we've found */
+			{
+			    assert(files_found < 64);
 
-			printf("block: %08x  file: %08x\n", blocks[block_index].single, nwos_ref_to_word(file_ref));
+			    copy_reference(&files[files_found].file, file_ref);
+			    files[files_found].seq = seq;
+
+			    extra_block_ptr = malloc(sizeof(extra_block));
+
+			    assert(extra_block_ptr != NULL);
+
+			    memcpy(extra_block_ptr, &extra_block, sizeof(extra_block));
+
+			    files[files_found].extra_block = extra_block_ptr;
+
+			    files_found++;
+			}
+
+			assert(blocks[block_index].extra_block == NULL);
+
+			blocks[block_index].extra_block = extra_block_ptr;
+
+			printf("block: %08x  extra_block: %p\n", blocks[block_index].single, blocks[block_index].extra_block);
 
 			blocks_found++;
 		    }
@@ -412,7 +512,7 @@ static bool check_file(ObjRef* file_ref)
 		    void_reference(&extra_block.next_block_ref);
 		}
 
-		seq++;
+		seq = (seq + 1) % NUM_STORED_SEQ;    /* wrap around if we've used all of them */
 	    }
 	}
 
@@ -422,161 +522,299 @@ static bool check_file(ObjRef* file_ref)
 	}
     }
 
-
-#if 0
-    ref_list_ptr = nwos_malloc_reference_list(&file_obj.block_list);
-
-    assert(num_blocks == ref_list_ptr->common_header.num_refs);
-
-//    printf("num_blocks: %u  block_list: %02x%02x%02x%02x\n", num_blocks,
-//	   file_obj.block_list.id[0],
-//	   file_obj.block_list.id[1],
-//	   file_obj.block_list.id[2],
-//	   file_obj.block_list.id[3]);
-
-    {
-    while (file_length > DATA_STORAGE_SIZE)   /* do all except the last block */
-    {
-	nwos_read_object_from_disk_and_decrypt(&ref_list_ptr->references[i], &data_obj, sizeof(data_obj), ivec, nwos_random_sequence[i%NUM_STORED_SEQ]);
-
-	    nwos_crc32_calculate((uint8*) &data_obj.storage, sizeof(data_obj.storage), chksum);
-
-	    if (memcmp(chksum, data_obj.checksum, sizeof(chksum)) != 0)
-	    {
-		printf("checksum error in file block - sequence: %d\n", i);
-		exit(1);
-	    }
-
-	    MD5Update(&md5_context, data_obj.storage, DATA_STORAGE_SIZE);    /* include this data in the md5 checksum */
-	    sha1_process_bytes(data_obj.storage, DATA_STORAGE_SIZE, &sha1_context);    /* and in the sha1 checksum */
-
-	    if (fwrite(data_obj.storage, sizeof(uint8), DATA_STORAGE_SIZE, fp) != DATA_STORAGE_SIZE)
-	    {
-		perror(path);
-		exit(1);
-	    }
-
-	    file_length -= DATA_STORAGE_SIZE;
-
-	    i++;
-	}
-
-	nwos_read_object_from_disk_and_decrypt(&ref_list_ptr->references[i], &data_obj, sizeof(data_obj), ivec, nwos_random_sequence[i%NUM_STORED_SEQ]);
-
-	nwos_crc32_calculate((uint8*) &data_obj.storage, file_length, chksum);
-
-	if (memcmp(chksum, data_obj.checksum, sizeof(chksum)) != 0)
-	{
-	    printf("checksum error in file block - sequence: %d\n", i);
-	    exit(1);
-	}
-
-	MD5Update(&md5_context, data_obj.storage, file_length);    /* include this data in the md5 checksum */
-	sha1_process_bytes(data_obj.storage, file_length, &sha1_context);    /* and in the sha1 checksum */
-
-	if (fwrite(data_obj.storage, sizeof(uint8), file_length, fp) != file_length)
-	{
-	    perror(path);
-	    exit(1);
-	}
-
-	if (fclose(fp) != 0)
-	{
-	    perror(path);
-	    exit(1);
-	}
-
-	if (result)   /* the file was ok, restore it's modification time */
-	{
-	    nwos_convert_time_stamp_to_timeval(assoc_obj.modification_time, &tv[1]);
-	    tv[0] = tv[1];  /* copy modification time into access time since we don't save the access time */
-	    utimes(path, tv);
-	}
-
-	MD5Final(md5_digest, &md5_context);   /* finish computing the md5 sum */
-	sha1_finish_ctx(&sha1_context, sha1_digest);  /* and the sha1 sum */
-
-	if (result)
-	{
-	    nwos_read_object_from_disk(&file_obj.md5sum, &md5_object, sizeof(md5_object));
-
-	    if (memcmp(md5_digest, md5_object.md5sum, sizeof(md5_digest)) == 0)
-	    {
-		int j;
-		printf("MD5 sum OK: ");
-		for (j = 0; j < 16; j++) printf("%02x", md5_digest[j]);
-		printf("\n");
-	    }
-	    else
-	    {
-		int j;
-		printf("MD5 checksum error, expected: ");
-		for (j = 0; j < 16; j++) printf("%02x", md5_object.md5sum[j]);
-		printf("\n                    received: ");
-		for (j = 0; j < 16; j++) printf("%02x", md5_digest[j]);
-		printf("\n");
-
-		result = false;
-	    }
-
-	    nwos_read_object_from_disk(&file_obj.sha1sum, &sha1_object, sizeof(sha1_object));
-
-	    if (memcmp(sha1_digest, sha1_object.sha1sum, sizeof(sha1_digest)) == 0)
-	    {
-		int j;
-		printf("SHA1 sum OK: ");
-		for (j = 0; j < 20; j++) printf("%02x", sha1_digest[j]);
-		printf("\n");
-	    }
-	    else
-	    {
-		int j;
-		printf("SHA1 checksum error, expected: ");
-		for (j = 0; j < 20; j++) printf("%02x", sha1_object.sha1sum[j]);
-		printf("\n                    received: ");
-		for (j = 0; j < 20; j++) printf("%02x", sha1_digest[j]);
-		printf("\n");
-
-		result = false;
-	    }
-	}
-
-	nwos_free_reference_list(ref_list_ptr);
-	ref_list_ptr = NULL;
-    }
-#endif
-
     return result;
 }
 
 
-bool scan_for_matching_blocks(char* compressed_file_name)
+/* this function takes the blocks that were found and figures out where to move them. */
+void find_new_locations()
+{
+    int i;
+    int j;
+    int k;
+    int in_a_row;
+    uint32 offset;
+    uint32 base;
+    uint32 space;
+    uint32 chunk_before;
+    uint32 space_before;
+    uint32 dist_before;
+    uint32 chunk;
+    uint32 chunk_after;
+    uint32 space_after;
+    uint32 dist_after;
+    uint32 new;
+    bool next_block;
+
+
+    in_a_row = 0;
+
+    for (i = 0; i < single_block_chunks; i++)
+    {
+	next_block = false;
+
+	if (blocks[i].first_block != NULL || blocks[i].extra_block != NULL)
+	{
+	    if (in_a_row == 0)
+	    {
+		chunk_before = (blocks[i].previous - RESERVED_PUBLIC_BLOCKS) / USABLE_BLOCKS_PER_CHUNK;
+		space_before = USABLE_BLOCKS_PER_CHUNK - (blocks[i].previous - RESERVED_PUBLIC_BLOCKS) % USABLE_BLOCKS_PER_CHUNK - 1;
+
+		chunk_after = 0;
+		space_after = 0;
+	    }
+
+	    chunk = (blocks[i].single - RESERVED_PUBLIC_BLOCKS) / USABLE_BLOCKS_PER_CHUNK;
+	    offset = (blocks[i].single - RESERVED_PUBLIC_BLOCKS) % USABLE_BLOCKS_PER_CHUNK;
+
+	    chunk_after = (blocks[i].subsequent - RESERVED_PUBLIC_BLOCKS) / USABLE_BLOCKS_PER_CHUNK;
+	    space_after = (blocks[i].subsequent - RESERVED_PUBLIC_BLOCKS) % USABLE_BLOCKS_PER_CHUNK;
+
+	    next_block = (i+1 < single_block_chunks && blocks[i].single == blocks[i+1].previous);
+	    if (next_block && blocks[i+1].first_block == NULL && blocks[i+1].extra_block == NULL)
+	    {
+		next_block = false;   /* the next chuck wasn't found */
+	    }
+
+	    printf("%08x %u (%u) - %u (%u) - %u (%u) %c\n",
+		   blocks[i].single, chunk_before, space_before, chunk, offset, chunk_after, space_after, next_block ? '+' : '-');
+
+	    if (next_block)
+	    {
+		in_a_row++;
+	    }
+	    else
+	    {
+		for (j = i - in_a_row; j <= i; j++)
+		{
+		    if (space_before == 0)       /* there is no space before */
+		    {
+			dist_before = 1;            /* always use after */
+			dist_after = 0;
+		    }
+		    else if (space_after == 0)   /* there is no space after */
+		    {
+			dist_before = 0;            /* always use after */
+			dist_after = 1;
+		    }
+		    else
+		    {
+			dist_before = blocks[j].single - blocks[i - in_a_row].previous;
+			dist_after = blocks[i].subsequent - blocks[j].single;
+		    }
+
+		    if (dist_before <= dist_after)
+		    {
+			base = RESERVED_PUBLIC_BLOCKS + (chunk_before + 1) * USABLE_BLOCKS_PER_CHUNK - space_before;
+			space = space_before;
+		    }
+		    else
+		    {
+			base = RESERVED_PUBLIC_BLOCKS + chunk_after * USABLE_BLOCKS_PER_CHUNK;
+			space = space_after;
+		    }
+
+		    new = base + random() % space;
+
+		    for (k = j; k > i - in_a_row; k--)
+		    {
+			if (blocks[k-1].new <= new) break;
+
+			blocks[k].new = blocks[k-1].new;
+		    }
+
+		    if (blocks[k-1].new == new)   /* duplicate */
+		    {
+			j--;   /* make it try this one again */
+		    }
+		    else
+		    {
+			blocks[k].new = new;
+		    }
+		}
+
+		for (j = i - in_a_row; j <= i; j++)
+		{
+		    chunk = (blocks[j].single - RESERVED_PUBLIC_BLOCKS) / USABLE_BLOCKS_PER_CHUNK;
+		    offset = (blocks[j].single - RESERVED_PUBLIC_BLOCKS) % USABLE_BLOCKS_PER_CHUNK;
+
+		    chunk_after = (blocks[j].new - RESERVED_PUBLIC_BLOCKS) / USABLE_BLOCKS_PER_CHUNK;
+		    space_after = (blocks[j].new - RESERVED_PUBLIC_BLOCKS) % USABLE_BLOCKS_PER_CHUNK;
+
+		    printf("   %d: %08x %u (%u) -> %08x %u (%u)\n", j, 
+			   blocks[j].single, chunk, offset,
+			   blocks[j].new, chunk_after, space_after);
+
+		    fflush(stdout);
+
+		    if (j > i - in_a_row)
+		    {
+			assert(blocks[j-1].new < blocks[j].new);
+		    }
+		    else
+		    {
+			assert(blocks[j].previous < blocks[j].new);
+		    }
+		}
+
+		in_a_row = 0;
+	    }
+	}
+    }
+}
+
+
+/* this function puts the new references in the reference list blocks that were found and. */
+void put_new_locations_in_ref_list()
+{
+    int i;
+    int j;
+
+    for (i = 0; i < single_block_chunks; i++)
+    {
+	if (blocks[i].first_block != NULL)
+	{
+	    for (j = 0; j < MAX_REFS_IN_REF_LIST; j++)
+	    {
+		if (nwos_ref_to_word(&blocks[i].first_block->refs[j]) == blocks[i].single) break;
+	    }
+
+	    assert(j < MAX_REFS_IN_REF_LIST);
+
+	    nwos_word_to_ref(blocks[i].new, &blocks[i].first_block->refs[j]);
+
+	    printf("first %p %d: %08x -> %08x\n", blocks[i].first_block, j, blocks[i].single, blocks[i].new);
+
+	    refs_changed++;
+	}
+	else if (blocks[i].extra_block != NULL)
+	{
+	    for (j = 0; j < MAX_REFS_IN_SIDECAR; j++)
+	    {
+		if (nwos_ref_to_word(&blocks[i].extra_block->refs[j]) == blocks[i].single) break;
+	    }
+
+	    assert(j < MAX_REFS_IN_SIDECAR);
+
+	    nwos_word_to_ref(blocks[i].new, &blocks[i].extra_block->refs[j]);
+
+	    blocks[i].extra_block->dirty = -1;
+
+	    printf("extra %p %d: %08x -> %08x\n", blocks[i].extra_block, j, blocks[i].single, blocks[i].new);
+
+	    refs_changed++;
+	}
+    }
+}
+
+
+extern BF_KEY blowfish_key;
+
+
+void encrypt_block(uint8 block[FILE_BLOCK_SIZE], uint8 ivec[IVEC_SIZE], uint8 seq_table[FILE_BLOCK_SIZE])
+{
+    uint8 buffer[FILE_BLOCK_SIZE];
+    int i;
+    int j;
+
+    for (i = 8; i < FILE_BLOCK_SIZE; i++)
+    {
+	/* find where the next byte is stored */
+	j = seq_table[i];    /* find where the next byte is stored */
+	buffer[j] = block[i];    /* save this byte there */
+    }
+
+    /* and encrypt the remainder 8 bytes at a time */
+    BF_cbc_encrypt((buffer + 8), (block + 8), (FILE_BLOCK_SIZE - 8), &blowfish_key, ivec, BF_ENCRYPT);
+}
+
+
+void encrypt_reference_lists()
+{
+    int i;
+    int j;
+    int seq;
+    int blocks_changed = 0;
+    Ref_List_Extra_Block* extra_block;
+    uint8 ivec[IVEC_SIZE];
+
+    for (i = 0; i < files_found; i++)
+    {
+	printf("file: %08x:\n", nwos_ref_to_word(&files[i].file));
+	fflush(stdout);
+
+	if (files[i].first_block != NULL)
+	{
+	    for (j = 0; j < MAX_REFS_IN_REF_LIST; j++)
+	    {
+		if (is_void_reference(&files[i].first_block->refs[j])) break;
+	    }
+
+	    assert(j > 0);
+
+	    nwos_crc32_calculate((uint8*) &files[i].first_block->refs, 
+				 (j + 1) * sizeof(ObjRef),
+				 files[i].first_block->list.common_header.data_chksum);
+
+	    memset(ivec, 0, sizeof(ivec));
+	    encrypt_block((uint8*)&files[i].first_block->list, ivec, nwos_random_sequence[0]);
+	    seq = 1;
+	    extra_block = files[i].first_block->next_block_ptr;
+
+	    printf("  first_block: %08x\n", nwos_ref_to_word(&files[i].first_block->list.common_header.id));
+	    blocks_changed++;
+	}
+	else
+	{
+	    seq = files[i].seq;
+	    extra_block = files[i].extra_block;
+	    memcpy(ivec, extra_block->ivec, sizeof(ivec));
+	}
+
+	while (extra_block != NULL)
+	{
+	    if (extra_block->dirty)
+	    {
+		for (j = 0; j < MAX_REFS_IN_SIDECAR; j++)
+		{
+		    if (is_void_reference(&extra_block->refs[j])) break;
+		}
+
+		assert(j > 0);
+
+		nwos_crc32_calculate((uint8*) &extra_block->refs, (j + 1) * sizeof(ObjRef), extra_block->checksum);
+
+		printf("  changed: %08x\n", nwos_ref_to_word(&extra_block->id));
+
+		extra_block->dirty = 0;
+	    }
+	    else
+	    {
+		printf("  unchanged: %08x\n", nwos_ref_to_word(&extra_block->id));
+	    }
+
+	    encrypt_block((uint8*)&extra_block->dirty, ivec, nwos_random_sequence[seq]);
+	    seq = (seq + 1) % NUM_STORED_SEQ;    /* wrap around if we've used all of them */
+	    extra_block = extra_block->next_block_ptr;
+	    blocks_changed++;
+	}
+
+	printf("blocks changed: %d\n", blocks_changed);
+	blocks_changed = 0;
+    }
+}
+
+
+bool scan_for_matching_blocks()
 {
-    uint8 big_key[16 + 8 + 4];
-    uint8 bf_key[16];
-    uint32 linear;
-    uint32 serial;
-    ObjRef root_object_ref;
     ObjRef object_class;
     C_struct_Class_Definition class_def_obj;
     ReferenceList* ref_list;
     int num_refs;
-    ObjRef file_ref;
     ObjRef file_class_ref;
     int i;
 
 
-    nwos_get_key_from_password(big_key, sizeof(big_key));
-
-    memcpy(bf_key, big_key, 16);
-    linear = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
-    memcpy(root_object_ref.id, big_key+20, 4);
-    serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
-
-    nwos_initialize_objectify(bf_key, linear, serial,   Compressed_File_RO, compressed_file_name);
-
-    nwos_set_root_object(&root_object_ref);
-
 
     assert(nwos_find_private_class_definition("FILE", &file_class_ref));
 
@@ -625,27 +863,204 @@ bool scan_for_matching_blocks(char* compressed_file_name)
 	ref_list = NULL;
     }
 
-    nwos_terminate_objectify();
-
     return false;  /* for now only do once */
 }
 
 
+void* find_block_in_ref_list(uint32 ref)
+{
+    int i;
+    Ref_List_Extra_Block* extra_block = NULL;
+
+    for (i = 0; i < files_found; i++)
+    {
+	extra_block = NULL;
+
+	if (files[i].first_block != NULL)
+	{
+	    if (nwos_ref_to_word(&files[i].first_block->list.common_header.id) == ref)
+	    {
+		return &files[i].first_block->list;
+	    }
+	    else
+	    {
+		extra_block = files[i].first_block->next_block_ptr;
+	    }
+	}
+	else
+	{
+	    extra_block = files[i].extra_block;
+	}
+
+	while (extra_block != NULL)
+	{
+	    if (nwos_ref_to_word(&extra_block->id) == ref)
+	    {
+		return &extra_block->dirty;
+	    }
+
+	    extra_block = extra_block->next_block_ptr;
+	}
+    }
+
+    return NULL;
+}
+
+
+void copy_file_with_changes(char* compressed_file_name, char* output_file_name)
+{
+    FILE* fp1;
+    FILE* fp2;
+    uint8 buf1[FILE_BLOCK_SIZE];
+    Disk_Header header1;
+    size_t read1;
+    size_t write2;
+    uint32 ref;
+    uint32 chunks_used;
+    int block_count;
+    int chunk_index;
+    uint8* block;
+
+
+    /* Open the old file and check it */
+
+    fp1 = fopen(compressed_file_name, "r");
+    if (fp1 == NULL)
+    {
+	perror(compressed_file_name);
+	exit(1);
+    }
+
+    read1 = fread(buf1, 1, sizeof(buf1), fp1);
+
+    if (read1 != FILE_BLOCK_SIZE)
+    {
+	if (ferror(fp1))
+	{
+	    perror(compressed_file_name);
+	}
+	else
+	{
+	    fprintf(stderr, "Unexpected end of file: %s\n", compressed_file_name);
+	}
+	fclose(fp1);
+	exit(1);
+    }
+
+    memcpy(&header1, buf1, sizeof(header1));
+
+    if (memcmp(header1.magic_number, "NWOS", 4) != 0)
+    {
+	fprintf(stderr, "Not an Objectify file: %s\n", compressed_file_name);
+	fclose(fp1);
+	exit(1);
+    }
+
+
+    /* modify the chunks used count in the header */
+
+    nwos_4_uint8_to_uint32(header1.used_chunks, &chunks_used);
+
+    printf("Changing used_chunks from %u to %u\n", chunks_used, chunks_used - blocks_found);
+
+    chunks_used -= blocks_found;
+
+    nwos_uint32_to_4_uint8(&chunks_used, header1.used_chunks);
+
+    memcpy(buf1, &header1, sizeof(header1));
+
+
+    /* Open the new file */
+
+    fp2 = fopen(output_file_name, "w");
+    if (fp2 == NULL)
+    {
+	perror(output_file_name);
+	exit(1);
+    }
+
+    write2 = fwrite(buf1, 1, sizeof(buf1), fp2);
+
+
+    read1 = fread(buf1, 1, sizeof(buf1), fp1);
+
+    block_count = 1;
+    ref = 0;
+
+    while (!feof(fp1) && read1 == FILE_BLOCK_SIZE)
+    {
+	if (buf1[0] != 0 || buf1[1] != 0 || buf1[2] != 0 || buf1[3] != 0)
+	{
+	    printf("\n%s block %d - first four bytes not zero: %02x%02x%02x%02x\n",
+		   compressed_file_name, block_count, buf1[0], buf1[1], buf1[2], buf1[3]);
+	    break;
+	}
+
+	nwos_4_uint8_to_uint32(&buf1[4], &ref);
+
+	/* first see if this is one of the blocks that moved */
+	chunk_index = find_single_block(ref);
+
+	if (chunk_index > -1 && blocks[chunk_index].new != 0)
+	{
+	    nwos_uint32_to_4_uint8(&blocks[chunk_index].new, &buf1[4]);
+
+	    write2 = fwrite(buf1, 1, sizeof(buf1), fp2);
+
+	    printf("Moved: %08x -> %02x%02x%02x%02x\n", ref, buf1[4], buf1[5], buf1[6], buf1[7]);
+	}
+	else
+	{
+	    block = find_block_in_ref_list(ref);
+
+	    if (block != NULL)
+	    {
+		write2 = fwrite(block, 1, FILE_BLOCK_SIZE, fp2);    /* write the newly encrypted block instead */
+
+		printf("Changed: %08x\n", ref);
+	    }
+	    else
+	    {
+		write2 = fwrite(buf1, 1, sizeof(buf1), fp2);
+	    }
+	}
+
+	
+	block_count++;
+
+	read1 = fread(buf1, 1, sizeof(buf1), fp1);
+    }
+
+    if (!feof(fp1) && read1 != FILE_BLOCK_SIZE)
+    {
+	perror(compressed_file_name);
+    }
+
+    fclose(fp1);
+
+    if (fclose(fp2) != 0)
+    {
+	perror(output_file_name);
+    }
+}
+
+
 int main(int argc, char* argv[])
 {
-    FILE* fp2;
-    FILE* fp3 = NULL;
-    uint8 buf2[FILE_BLOCK_SIZE];
-    int block2;
-    size_t read2;
-    size_t write3;
-    Disk_Header header2;
-    bool wrapped_file = false;     /* true if older file that wasn't sequencial */
-    int header1_version;
-    int header2_version;
-    uint32 ref_offset;
-    uint32 new_ref;
-    uint32 new_chunk;
+    uint8 big_key[16 + 8 + 4];
+    uint8 bf_key[16];
+    uint32 linear;
+    uint32 serial;
+    ObjRef root_object_ref;
+    ObjRef assoc_class_ref;
+    ObjRef object_class;
+    C_struct_Class_Definition class_def_obj;
+    C_struct_Path_And_File_Association assoc_obj;
+    ReferenceList* ref_list;
+    int num_refs;
+    int i;
+    int j;
+    char path[128];
 
     if (argc != 3)
     {
@@ -655,210 +1070,86 @@ int main(int argc, char* argv[])
 
     nwos_log_arguments(argc, argv);
 
+    nwos_get_key_from_password(big_key, sizeof(big_key));
+
+    memcpy(bf_key, big_key, 16);
+    linear = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
+    memcpy(root_object_ref.id, big_key+20, 4);
+    serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
+
+    /* read the compressed file and find the chunks with only one block */
     find_single_block_chunks(argv[1]);
 
-    scan_for_matching_blocks(argv[1]);
+    /* fire up objectify and see which of those single blocks we can find in the files */
+    nwos_initialize_objectify(bf_key, linear, serial,   Compressed_File_RO, argv[1]);
 
-#if 0
-    fp3 = fopen(argv[3], "w");
+    nwos_set_root_object(&root_object_ref);
 
-    if (fp3 == NULL)
+    scan_for_matching_blocks();
+
+    /* find new locations for the blocks we matched up */
+    find_new_locations();
+    put_new_locations_in_ref_list();
+
+    /* and re-encrypt the modified reference list blocks */
+    encrypt_reference_lists();
+
+    /* shut down objectify for now */
+    nwos_terminate_objectify();
+
+    /* copy the file making the changes as we go */
+    copy_file_with_changes(argv[1], argv[2]);
+
+    printf("\nVerifying files:\n");
+    fflush(stdout);
+
+    /* export the modified files */
+    nwos_initialize_objectify(bf_key, linear, serial,   Compressed_File_RO, argv[2]);
+
+    nwos_set_root_object(&root_object_ref);
+
+    assert(nwos_find_private_class_definition("PATH AND FILE ASSOCIATION", &assoc_class_ref));
+    
+    nwos_read_class_definition(&assoc_class_ref, &class_def_obj);
+
+    ref_list = nwos_malloc_reference_list(&class_def_obj.header.object.references);
+
+    num_refs = ref_list->common_header.num_refs;
+
+    printf("num_refs: %d\n", num_refs);
+
+    for (i = 0; i < num_refs; i++)
     {
-	perror(argv[3]);
-	fclose(fp1);
-	fclose(fp2);
-	exit(1);
-    }
+	nwos_get_object_class(&ref_list->references[i], &object_class);
 
-    write3 = fwrite(buf2, 1, sizeof(buf2), fp3);
-
-    if (write3 != FILE_BLOCK_SIZE)
-    {
-	perror(argv[3]);
-	fclose(fp1);
-	fclose(fp2);
-	fclose(fp3);
-	exit(1);
-    }
-
-	    ref_offset = (ref - RESERVED_PUBLIC_BLOCKS) % USABLE_BLOCKS_PER_CHUNK;
-	    new_ref = RESERVED_PUBLIC_BLOCKS + USABLE_BLOCKS_PER_CHUNK * chunk + random() % ref_offset;
-
-	    new_chunk = (new_ref - RESERVED_PUBLIC_BLOCKS) / USABLE_BLOCKS_PER_CHUNK;
-
-	    assert(prev_ref1 < new_ref && new_ref < ref);
-	    assert(new_chunk == chunk);
-	    assert((new_ref - RESERVED_PUBLIC_BLOCKS) % USABLE_BLOCKS_PER_CHUNK < ref_offset);
-
-
-    while(something)
-    {
-	if (buf1[4] == buf2[4] && buf1[5] == buf2[5] && buf1[6] == buf2[6] && buf1[7] == buf2[7])
+	if (is_same_object(&object_class, &assoc_class_ref))
 	{
-	    /* blocks have the same id number, must have changed */
+	    nwos_read_object_from_disk(&ref_list->references[i], &assoc_obj, sizeof(assoc_obj));
 
-	    if (memcmp(buf1, buf2, FILE_BLOCK_SIZE) != 0)
+	    for (j = 0; j < files_found; j++)
 	    {
-		printf("%02x%02x%02x%02x: changed\n",
-		       buf1[4], buf1[5], buf1[6], buf1[7]);
-
-		write3 = fwrite(buf2, 1, sizeof(buf2), fp3);
-	    }
-	    else   /* block stayed the same write the old block */
-	    {
-		write3 = fwrite(buf1, 1, sizeof(buf1), fp3);
-	    }
-
-	    if (write3 != FILE_BLOCK_SIZE)
-	    {
-		perror(argv[2]);
-		fclose(fp1);
-		fclose(fp2);
-		fclose(fp3);
-		exit(1);
-	    }
-
-	    read1 = fread(buf1, 1, sizeof(buf1), fp1);
-	    block1++;
-	    read2 = fread(buf2, 1, sizeof(buf2), fp2);
-	    block2++;
-	}
-	else   /* id is different, assume this an add, since right now can't delete */
-	{
-	    if (wrapped_file)
-	    {
-		/* if we've found the proper place to insert it */
-		if (buf1[4] == buf2[0] && buf1[5] == buf2[1] && buf1[6] == buf2[2] && buf1[7] == buf2[3])
+		if (is_same_object(&assoc_obj.file, &files[j].file))
 		{
-		    printf("%02x%02x%02x%02x: added\n",
-			   buf2[4], buf2[5], buf2[6], buf2[7]);
-
-		    buf2[0] = 0;
-		    buf2[1] = 0;
-		    buf2[2] = 0;
-		    buf2[3] = 0;
-
-		    write3 = fwrite(buf2, 1, sizeof(buf2), fp3);
-
-		    read2 = fread(buf2, 1, sizeof(buf2), fp2);
-		    block2++;
+		    nwos_file_path_to_string(&assoc_obj.path, path, sizeof(path));
+		    printf("Restoring file: %08x - %s\n", nwos_ref_to_word(&files[j].file), path);
+		    nwos_restore_file(&ref_list->references[i], path);
+		    break;
 		}
-		else  /* not time to insert yet, just write the old block */
-		{
-		    write3 = fwrite(buf1, 1, sizeof(buf1), fp3);
-
-		    read1 = fread(buf1, 1, sizeof(buf1), fp1);
-		    block1++;
-		}
-	    }
-	    else   /* new non-wrapped file that is sequential */
-	    {
-		/* if we've found the proper place to insert it */
-		if (buf1[4] > buf2[4] || 
-		    (buf1[4] == buf2[4] && buf1[5] > buf2[5]) ||
-		    (buf1[4] == buf2[4] && buf1[5] == buf2[5] && buf1[6] > buf2[6]) ||
-		    (buf1[4] == buf2[4] && buf1[5] == buf2[5] && buf1[6] == buf2[6] && buf1[7] > buf2[7])
-		    )
-		{
-		    printf("%02x%02x%02x%02x: added\n",
-			   buf2[4], buf2[5], buf2[6], buf2[7]);
-
-		    write3 = fwrite(buf2, 1, sizeof(buf2), fp3);
-
-		    read2 = fread(buf2, 1, sizeof(buf2), fp2);
-		    block2++;
-		}
-		else  /* not time to insert yet, just write the old block */
-		{
-		    write3 = fwrite(buf1, 1, sizeof(buf1), fp3);
-
-		    read1 = fread(buf1, 1, sizeof(buf1), fp1);
-		    block1++;
-		}
-	    }
-
-	    if (write3 != FILE_BLOCK_SIZE)
-	    {
-		perror(argv[3]);
-		fclose(fp1);
-		fclose(fp2);
-		fclose(fp3);
-		exit(1);
 	    }
 	}
     }
 
-    /* finish writing output */
+    nwos_terminate_objectify();
 
-    while (!feof(fp1) && read1 == FILE_BLOCK_SIZE)
-    {
-	write3 = fwrite(buf1, 1, sizeof(buf1), fp3);
 
-	if (write3 != FILE_BLOCK_SIZE)
-	{
-	    perror(argv[3]);
-	    fclose(fp1);
-	    fclose(fp2);
-	    fclose(fp3);
-	    exit(1);
-	}
-
-	read1 = fread(buf1, 1, sizeof(buf1), fp1);
-	block1++;
-    }
-
-    /* more in the patch file */
-
-    while (!feof(fp2) && read2 == FILE_BLOCK_SIZE)
-    {
-	printf("%02x%02x%02x%02x: added\n", buf2[4], buf2[5], buf2[6], buf2[7]);
-
-	write3 = fwrite(buf2, 1, sizeof(buf2), fp3);
-
-	if (write3 != FILE_BLOCK_SIZE)
-	{
-	    perror(argv[3]);
-	    fclose(fp1);
-	    fclose(fp2);
-	    fclose(fp3);
-	    exit(1);
-	}
-
-	read2 = fread(buf2, 1, sizeof(buf2), fp2);
-	block1++;
-    }
-#endif
     printf("\n");
 
-#if 0
-    if (memcmp(header1.version_string, header2.version_string, 4) != 0)
-    {
-	fprintf(stderr, "WARNING, versions don't match - %s: %c%c%c%c  %s: %c%c%c%c\n",
-		argv[1], 
-		header1.version_string[0], header1.version_string[1],
-		header1.version_string[2], header1.version_string[3],
-		argv[2], 
-		header2.version_string[0], header2.version_string[1],
-		header2.version_string[2], header2.version_string[3]);
-    }
-
-    if (!feof(fp2) && read2 != FILE_BLOCK_SIZE)
-    {
-	perror(argv[2]);
-    }
-
-    fclose(fp2);
-
-    if (fclose(fp3) != 0)
-    {
-	perror(argv[3]);
-    }
-#endif
-
     printf("Total chunks: %d\n", total_chunks);
     printf("Single block chunks: %d\n", single_block_chunks);
     printf("Files checked: %d\n", files_checked);
+    printf("Files found: %d\n", files_found);
     printf("Blocks found: %d\n", blocks_found);
+    printf("Refs changed: %d\n", refs_changed);
     printf("First block found: %d\n", first_block_blocks);
     printf("Extra block found: %d\n", extra_block_blocks);
 
