@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                 Copyright (C) 2005-2007  QRW Software
+--                 Copyright (C) 2005-2008  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,14 @@
 --
 --
 -- $Log: class_definition.c,v $
+-- Revision 1.28  2008/07/19 13:07:03  jsedwards
+-- Added asserts to verify that after an updated class is cloned that it can
+-- be accessed succesfully.  NO changes to the code operation.
+--
+-- Revision 1.27  2008/07/17 12:07:18  jsedwards
+-- Changed nwos_find_or_create_private_class_definition to return
+-- ObjCreateResult instead of void.
+--
 -- Revision 1.26  2008/03/15 16:41:00  jsedwards
 -- Changed 'nwos_find_private_class_definition' to call new
 -- 'nwos_get_object_class_without_update function' instead of
@@ -346,11 +354,13 @@ void nwos_clone_updated_class_definition(ObjRef* old_private_ref, ObjRef* clone_
 /* This routine finds or creates a private class definition with the given name. */
 /*********************************************************************************/
 
-void nwos_find_or_create_private_class_definition(char* name_of_class, ObjRef* ref)
+ObjCreateResult nwos_find_or_create_private_class_definition(char* name_of_class, ObjRef* ref)
 {
     ObjRef public_ref;
     ObjRef old_private_ref;
+    ObjRef new_private_ref;
     EveryObject header;
+    ObjCreateResult result = FOUND_EXISTING;
 
     if (nwos_find_private_class_definition(name_of_class, ref))   /* see if public is updated */
     {
@@ -361,7 +371,20 @@ void nwos_find_or_create_private_class_definition(char* name_of_class, ObjRef* r
 	if (!is_void_reference(&header.object.next_version))  /* the public class has been updated */
 	{
 	    copy_reference(&old_private_ref, ref);
+
 	    nwos_clone_updated_class_definition(&old_private_ref, ref);
+	    assert(!is_same_object(&old_private_ref, ref));
+
+	    assert(nwos_find_private_class_definition(name_of_class, &new_private_ref));
+	    assert(is_same_object(&new_private_ref, ref));
+
+	    nwos_read_object_headers_from_disk(ref, &header);
+
+	    nwos_read_object_headers_from_disk(&header.object.clone_of, &header);
+
+	    assert(is_void_reference(&header.object.next_version));
+
+	    result = CREATED_NEW;
 	}
     }
     else
@@ -370,7 +393,11 @@ void nwos_find_or_create_private_class_definition(char* name_of_class, ObjRef* r
 	assert(nwos_find_public_class_definition(name_of_class, &public_ref));
 
 	nwos_clone_class_definition(&public_ref, ref);
+
+	result = CREATED_NEW;
     }
+
+    return result;
 }
 
 
