@@ -27,6 +27,12 @@
 --
 --
 -- $Log: disk_io.c,v $
+-- Revision 1.89  2008/08/10 15:16:38  jsedwards
+-- Added printing of path in "Missing magic number" error message.
+--
+-- Revision 1.88  2008/08/10 15:07:26  jsedwards
+-- Added path in which error occurred to "Incorrect version string" message.
+--
 -- Revision 1.87  2008/07/19 13:34:12  jsedwards
 -- Added new disk_io_is_read_only and disk_io_is_read_write functions.
 --
@@ -743,13 +749,13 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
     if (memcmp(nwos_disk_header.magic_number, MAGIC_NUMBER, 4) != 0)
     {
-	fprintf(stderr, "Missing magic number in disk header\n");
+	fprintf(stderr, "Missing magic number in disk header: %s\n", nwos_public_path);
 	exit(1);
     }
 
     if (memcmp(nwos_disk_header.version_string, VERSION_STRING, 4) != 0)
     {
-	fprintf(stderr, "Incorrect version string in disk header\n");
+	fprintf(stderr, "Incorrect version string in disk header: %s\n", nwos_public_path);
 	exit(1);
     }
 
@@ -820,7 +826,7 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
 	if (memcmp(nwos_disk_header.magic_number, MAGIC_NUMBER, 4) != 0)
 	{
-	    fprintf(stderr, "Missing magic number in disk header\n");
+	    fprintf(stderr, "Missing magic number in disk header: %s\n", nwos_private_path);
 	    exit(1);
 	}
 
@@ -831,7 +837,7 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 	    if ((compressed_file && memcmp(nwos_disk_header.version_string, OLDEST_COMPATIBLE_COMPRESSED_VERSION, 4) < 0) ||
 		(!compressed_file && memcmp(nwos_disk_header.version_string, VERSION_STRING, 4) != 0))
 	    {
-		fprintf(stderr, "Incorrect version string in disk header\n");
+		fprintf(stderr, "Incorrect version string in disk header: %s\n", nwos_private_path);
 		exit(1);
 	    }
 	}
