@@ -27,6 +27,10 @@
 --
 --
 -- $Log: disk_usage.c,v $
+-- Revision 1.24  2008/08/10 15:23:25  jsedwards
+-- Added path to "Missing magic number" and "Incorrect version in header" error
+-- messages.
+--
 -- Revision 1.23  2008/04/02 03:06:21  jsedwards
 -- Added file locking.
 --
@@ -324,13 +328,13 @@ int main(int argc, char* argv[])
 
     if (memcmp(disk_header.magic_number, MAGIC_NUMBER, 4) != 0)
     {
-	fprintf(stderr, "Missing magic number in disk header\n");
+	fprintf(stderr, "Missing magic number in disk header: %s\n", obj_file_path);
 	exit(1);
     }
 
     if (memcmp(disk_header.version_string, VERSION_STRING, 4) != 0)
     {
-	fprintf(stderr, "Incorrect version string in disk header\n");
+	fprintf(stderr, "Incorrect version string in disk header: %s\n", obj_file_path);
 	exit(1);
     }
 
