@@ -27,6 +27,10 @@
 --
 --
 -- $Log: objectify.h,v $
+-- Revision 1.146  2008/08/03 15:34:30  jsedwards
+-- Added DISC_LIST_* flags and masks and changed nwos_read_files_disc_list to
+-- return uint32 instead of int.
+--
 -- Revision 1.145  2008/07/24 16:43:16  jsedwards
 -- Added external declaration for nwos_read_files_disc_list.
 --
@@ -1098,9 +1102,19 @@ extern void nwos_check_directory_is_all_ok(const char* path);
 
 #define MAX_FILES_PER_DISC_LIST 1000000
 #define MAX_SIZE_DISC_LIST (sizeof(C_struct_Disc_List) + sizeof(ObjRef) * MAX_FILES_PER_DISC_LIST)
+
+/* the following defines are for the return value from the nwos_read_files_disc_list function */
+#define DISC_LIST_NUM_FILES_MASK       0x0FFFFFFF
+#define DISC_LIST_ERROR_SHIFT          28
+#define DISC_LIST_EMPTY_DIR_FLAG       (1 << DISC_LIST_ERROR_SHIFT)
+#define DISC_LIST_EMPTY_FILE_FLAG      (2 << DISC_LIST_ERROR_SHIFT)
+#define DISC_LIST_INVALID_TYPE_FLAG    (4 << DISC_LIST_ERROR_SHIFT)
+#define DISC_LIST_TOO_MANY_FILES_FLAG  (8 << DISC_LIST_ERROR_SHIFT)
+#define DISC_LIST_ERROR_MASK           (DISC_LIST_EMPTY_DIR_FLAG | DISC_LIST_EMPTY_FILE_FLAG | DISC_LIST_INVALID_TYPE_FLAG | DISC_LIST_TOO_MANY_FILES_FLAG)
+
 extern size_t nwos_get_disc_list_object_size(void* disc_list_obj);
 extern bool nwos_find_disc_list(char id[12], ObjRef* ref);
-extern int nwos_read_files_disc_list(const char* root_dir, const char* subdirectory, char* file_names[MAX_FILES_PER_DISC_LIST], int num_files);
+extern uint32 nwos_read_files_disc_list(const char* root_dir, const char* subdirectory, char* file_names[MAX_FILES_PER_DISC_LIST], uint32 num_files);
 extern bool nwos_find_matching_disc_list(ObjRef files[MAX_FILES_PER_DISC_LIST], ObjRef* ref);
 extern ObjCreateResult nwos_create_disc_list(char id[12], ObjRef disc_list[MAX_FILES_PER_DISC_LIST], ObjRef* ref);
 extern bool nwos_find_disc_copy(ObjRef* disc_list, int copy_num, ObjRef* ref);
