@@ -27,6 +27,21 @@
 --
 --
 -- $Log: prep_disk.c,v $
+-- Revision 1.57  2008/07/19 15:40:20  jsedwards
+-- Removed the #ifndef TEST_PASS_PHRASE around the verifying it that it is
+-- okay to write over the storage.  Too dangerous!!
+--
+-- Revision 1.56  2008/07/19 11:17:08  jsedwards
+-- Moved start_progress_bar, update_progress_bar, and finish_progress_bar
+-- functions to their own file: progress_bar.c and renamed with nwos_ prefix.
+--
+-- Revision 1.55  2008/04/02 03:06:21  jsedwards
+-- Added file locking.
+--
+-- Revision 1.54  2008/03/12 14:47:07  jsedwards
+-- Added code to create all subdirectories in the path before writing the
+-- private objects file.
+--
 -- Revision 1.53  2008/02/03 20:38:22  jsedwards
 -- Change to pass DEFAULT_FILE to nwos_initialize_objectify instead of
 -- private_path because passing a file now to initialize tells it that it
@@ -254,60 +269,6 @@
 
 
 
-/***************************************/
-/*  Functions to print a progress bar  */
-/***************************************/
-
-static uint progress_bar_percent;
-
-static void start_progress_bar()
-{
-    progress_bar_percent = 0;
-
-    printf("Percent complete: ");
-    fflush(stdout);
-}
-
-
-static void update_progress_bar(float percent)  /* 0.0 <= percent < 1.0 */
-{
-    while (percent >= (float)progress_bar_percent / 100)
-    {
-	if (progress_bar_percent % 10 == 0)
-	{
-	    printf("%d", progress_bar_percent);
-	}
-	else
-	{
-	    printf(".");
-	}
-	fflush(stdout);
-
-	progress_bar_percent += 2;
-    }
-}
-
-
-static void finish_progress_bar()
-{
-    while (progress_bar_percent <= 100)
-    {
-	if (progress_bar_percent % 10 == 0)
-	{
-	    printf("%d", progress_bar_percent);
-	}
-	else
-	{
-	    printf(".");
-	}
-
-	progress_bar_percent += 2;
-    }
-
-    printf("\n");
-}
-
-
 /**********************************************************/
 /* Determine if path is a regular file or a block device. */
 /**********************************************************/
@@ -561,13 +522,13 @@ uint32 write_chunks(int fd, uint8* chunk, uint32 num_chunks)
     size_t bytes_written;
     uint32 blocks = 0;
 
-    start_progress_bar();
+    nwos_start_progress_bar();
 
     errno = 0;
 
     while (num_chunks == 0 || blocks < (num_chunks * BLOCKS_IN_CHUNK))
     {
-	update_progress_bar((float)blocks / (float)(num_chunks * BLOCKS_IN_CHUNK));
+	nwos_update_progress_bar((float)blocks / (float)(num_chunks * BLOCKS_IN_CHUNK));
 
 	bytes_written = write(fd, chunk, CHUNK_SIZE);
 
@@ -594,7 +555,7 @@ uint32 write_chunks(int fd, uint8* chunk, uint32 num_chunks)
 	exit(1);
     }
 
-    finish_progress_bar();
+    nwos_finish_progress_bar();
 
     return blocks;
 }
@@ -630,6 +591,7 @@ int main(int argc, char* argv[])
 {
     int obj_fd;
     const char* private_path;
+    struct flock lock;
     uint8 *chunk;
     char answer[8];
     int arg_index = 1;
@@ -652,6 +614,7 @@ int main(int argc, char* argv[])
     uint32 linear;
     uint32 serial;
     char msg[128];
+    char* p;
 #if 0
     ObjRef root_object_reference;
     bool upgrade = false;
@@ -672,6 +635,26 @@ int main(int argc, char* argv[])
     if (!is_device(private_path))
     {
 	is_file = true;
+
+	/* NOTE: this is a bad thing to do, it temporarily modifies */
+	/* the string returned by nwos_get_private_objects_path.    */
+        /* We cheat the system by calling strchr.                   */
+
+	p = strchr(private_path, '/');
+	assert(p != NULL);
+	p = strchr(p + 1, '/');
+	while (p != NULL)
+	{
+	    *p = '\0';
+	    if (mkdir(private_path, 0755) != 0 && errno != EEXIST)
+	    {
+		perror(p);
+		exit(1);
+	    }
+	    *p = '/';
+
+	    p = strchr(p + 1, '/');
+	}
     }
 
     if (argc > 1 && *argv[arg_index] == '-')   /* option specified */
@@ -758,7 +741,6 @@ int main(int argc, char* argv[])
 	exit(1);
     }
 
-#ifndef TEST_PASS_PHRASE
     printf("\n");
     printf("WARNING: all of the contents of %s will be ERASED!!!\n", private_path);
     printf("\n");
@@ -774,7 +756,6 @@ int main(int argc, char* argv[])
     {
 	exit(1);
     }
-#endif
 
     /***************************/
     /* Open the storage device */
@@ -796,6 +777,19 @@ int main(int argc, char* argv[])
     }
 
 
+    lock.l_type = F_WRLCK;
+    lock.l_whence = SEEK_SET;
+    lock.l_start = 0;
+    lock.l_len = 0;
+
+    if (fcntl(obj_fd, F_SETLK, &lock) != 0)
+    {
+	perror(private_path);
+	exit(1);
+    }
+
+
+
     /*********************************/
     /* finally write the disk header */
     /*********************************/
@@ -906,9 +900,9 @@ int main(int argc, char* argv[])
 
 	if (allocate_all)
 	{
-	    start_progress_bar();
-	    nwos_allocate_all_chunks_hack(nwos_ref_to_word(&private_root_obj_ref), &update_progress_bar);
-	    finish_progress_bar();
+	    nwos_start_progress_bar();
+	    nwos_allocate_all_chunks_hack(nwos_ref_to_word(&private_root_obj_ref), &nwos_update_progress_bar);
+	    nwos_finish_progress_bar();
 	}
 	else
 	{
