@@ -27,6 +27,19 @@
 --
 --
 -- $Log: reference_list.c,v $
+-- Revision 1.36  2008/08/02 04:31:25  jsedwards
+-- Finished changes to read a reference list that has the old encryption at
+-- the beginning and later on has blocks with the new encryption.
+--
+-- Revision 1.35  2008/08/01 22:10:53  jsedwards
+-- Changed to check both the class definition and the checksum instead of just
+-- the class definition.  Also fixed to not re-read the block if it is a public
+-- reference list.
+--
+-- Revision 1.34  2008/08/01 00:13:18  jsedwards
+-- Changed to try old decryption algorithm if class definition doesn't match
+-- nwos_reference_list_class_ref after reading with new decryption algorithm.
+--
 -- Revision 1.33  2008/07/19 13:59:15  jsedwards
 -- Changed nwos_add_to_references to call nwos_get_object_class_without_update
 -- instead of nwos_get_object_class.  This fixed a problem where cloning the
@@ -182,7 +195,7 @@ static int ref_list_tick = 0;
 static Block_Cache ref_list_cache[REF_LIST_CACHE_SIZE];
 
 
-static void check_ref_list_data(ObjRef* ref, uint8* data, size_t data_size, uint8 stored[4])
+static bool ref_list_checksum_matches(uint8* data, size_t data_size, uint8 stored[4])
 {
     uint8 computed[4];
 
@@ -202,24 +215,7 @@ static void check_ref_list_data(ObjRef* ref, uint8* data, size_t data_size, uint
 
     nwos_crc32_calculate((uint8*)data, data_size, computed);
 
-    if (stored[0] != computed[0] || stored[1] != computed[1] || stored[2] != computed[2] || stored[3] != computed[3])
-    {
-	printf("reference list: %02x%02x%02x%02x - bad data checksum, ", ref->id[0], ref->id[1], ref->id[2], ref->id[3]);
-	printf("size %zd - ", data_size);
-	printf("computed %02x%02x%02x%02x - ", computed[0], computed[1], computed[2], computed[3]);
-	printf("stored %02x%02x%02x%02x\n", stored[0], stored[1], stored[2], stored[3]);
-
-	printf("\n");
-	{
-	  int i;
-	  for (i = 0; i < data_size; i++)
-	  {
-	      printf("%02x%c", data[i], (i % 16) == 15 ? '\n' : ' ');
-	  }
-	  printf("\n");
-	}
-	exit(1);
-    }
+    return stored[0] == computed[0] && stored[1] == computed[1] && stored[2] == computed[2] && stored[3] == computed[3];
 }
 
 
@@ -230,12 +226,12 @@ static void read_ref_list_into_cache(Block_Cache* cache)
     Ref_List_Extra_Block* next_ptr;            /* points to the next block */
     int i;
     int seq;
+    bool use_old_decryption = false;
+    int save_num_refs;
 
     assert(sizeof(Ref_List_First_Block) == FILE_BLOCK_SIZE + sizeof(void*) + IVEC_SIZE);
     assert(sizeof(Ref_List_Extra_Block) == FILE_BLOCK_SIZE + sizeof(void*) + IVEC_SIZE);
 
-    memset(ivec, 0, sizeof(ivec));
-
     cache->first_block = nwos_malloc(sizeof(Ref_List_First_Block));
     assert(cache->first_block != NULL);
 
@@ -244,26 +240,51 @@ static void read_ref_list_into_cache(Block_Cache* cache)
 
     cache->first_block->next_block_ptr = NULL;
 
-    assert(nwos_read_object_from_disk_and_decrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[0]));
-
-    assert(is_same_object(&cache->first_block->list.common_header.class_definition, &nwos_reference_list_class_ref));
-
-    cache->num_refs = 0;
-
-    while (cache->num_refs < MAX_REFS_IN_REF_LIST)
+    while (1)
     {
-	if (is_void_reference(&cache->first_block->refs[cache->num_refs])) break;
-	cache->num_refs++;
+	memset(ivec, 0, sizeof(ivec));
+
+	if (use_old_decryption)
+	{
+	    assert(nwos_read_object_from_disk_and_old_decrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[0]));
+	}
+	else
+	{
+	    assert(nwos_read_object_from_disk_and_decrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[0]));
+	}
+
+	if (nwos_reference_type(&cache->ref) == Public_Reference || is_same_object(&cache->first_block->list.common_header.class_definition, &nwos_reference_list_class_ref))
+	{
+	    cache->num_refs = 0;
+
+	    while (cache->num_refs < MAX_REFS_IN_REF_LIST)
+	    {
+		if (is_void_reference(&cache->first_block->refs[cache->num_refs])) break;
+		cache->num_refs++;
+	    }
+
+	    if (ref_list_checksum_matches((uint8*) cache->first_block->refs, (cache->num_refs + 1) * sizeof(ObjRef), cache->first_block->list.common_header.data_chksum))
+	    {
+		break;   /* found correct decryption, exit loop */
+	    }
+
+	    if (nwos_reference_type(&cache->ref) == Public_Reference || use_old_decryption)
+	    {
+		printf("\n");
+		fflush(stdout);
+		fprintf(stderr, "Unable to decrypt reference list: %08x\n", nwos_ref_to_word(&cache->ref));
+		nwos_terminate_objectify();
+		exit(1);
+	    }
+	}
+
+	use_old_decryption = true;
     }
 
+
     /* use the flags as a pointer to the next block */
     prev_ptr = (Ref_List_Extra_Block*) cache->first_block;
 
-    check_ref_list_data(&cache->ref,
-			(uint8*) cache->first_block->refs,
-			(cache->num_refs + 1) * sizeof(ObjRef),                 /* include void or next_block_ref */
-			cache->first_block->list.common_header.data_chksum);
-
     if (cache->num_refs < MAX_REFS_IN_REF_LIST)
     {
 	prev_ptr->next_block_ref.word = 0;
@@ -283,15 +304,45 @@ static void read_ref_list_into_cache(Block_Cache* cache)
 
 	memcpy(next_ptr->ivec, ivec, sizeof(next_ptr->ivec));  /* save this ivec in case this block has to be written back to disk */
 
-	assert(nwos_read_object_from_disk_and_decrypt(&prev_ptr->next_block_ref, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[seq]));
+	save_num_refs = cache->num_refs;
 
-	for (i = 0; i < MAX_REFS_IN_SIDECAR; i++)
+	while (1)
 	{
-	    if (is_void_reference(&next_ptr->refs[i])) break;
-	    cache->num_refs++;
-	}
+	    memcpy(ivec, next_ptr->ivec, sizeof(ivec));  /* get the saved ivec back */
 
-	check_ref_list_data(&cache->ref, (uint8*) next_ptr->refs, (i + 1) * sizeof(ObjRef), next_ptr->checksum);
+	    if (use_old_decryption)
+	    {
+		assert(nwos_read_object_from_disk_and_old_decrypt(&prev_ptr->next_block_ref, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[seq]));
+	    }
+	    else
+	    {
+		assert(nwos_read_object_from_disk_and_decrypt(&prev_ptr->next_block_ref, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[seq]));
+	    }
+
+	    cache->num_refs = save_num_refs;
+
+	    for (i = 0; i < MAX_REFS_IN_SIDECAR; i++)
+	    {
+		if (is_void_reference(&next_ptr->refs[i])) break;
+		cache->num_refs++;
+	    }
+
+	    if (ref_list_checksum_matches((uint8*) next_ptr->refs, (i + 1) * sizeof(ObjRef), next_ptr->checksum))
+	    {
+		break;
+	    }
+
+	    if (nwos_reference_type(&cache->ref) == Public_Reference || !use_old_decryption)
+	    {
+		printf("\n");
+		fflush(stdout);
+		fprintf(stderr, "Unable to decrypt reference list: %08x  seq: %d\n", nwos_ref_to_word(&cache->ref), seq);
+		nwos_terminate_objectify();
+		exit(1);
+	    }
+
+	    use_old_decryption = false;   /* could have encountered a new one */
+	}
 
 	if (i < MAX_REFS_IN_SIDECAR)    /* the link will be garbage, make it void */
 	{
